@@ -54,6 +54,10 @@ def nfvswitch_config_path():
     return "/etc/sysconfig/nfvswitch"
 
 
+def vpp_config_path():
+    return "/etc/vpp/startup.conf"
+
+
 def route_config_path(name):
     return "/etc/sysconfig/network-scripts/route-%s" % name
 
@@ -116,6 +120,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         self.linuxbond_data = {}
         self.ib_interface_data = {}
         self.linuxteam_data = {}
+        self.vpp_interface_data = {}
         self.member_names = {}
         self.renamed_interfaces = {}
         self.bond_primary_ifaces = {}
@@ -294,6 +299,20 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             data += "DEVICETYPE=ovs\n"
             data += "TYPE=OVSDPDKPort\n"
             data += "OVS_BRIDGE=%s\n" % base_opt.bridge_name
+            # Validation of DPDK port having only one interface is done prior
+            # to this. So accesing the interface name statically.
+            # Also pci_address would be valid here, since
+            # bind_dpdk_interfaces() is invoked before this.
+            pci_address = utils.get_stored_pci_address(
+                base_opt.members[0].name, self.noop)
+            ovs_extra.append("set Interface $DEVICE options:dpdk-devargs="
+                             "%s" % pci_address)
+            if base_opt.mtu:
+                ovs_extra.append("set Interface $DEVICE mtu_request=$MTU")
+            if base_opt.rx_queue:
+                data += "RX_QUEUE=%i\n" % base_opt.rx_queue
+                ovs_extra.append("set Interface $DEVICE " +
+                                 "options:n_rxq=$RX_QUEUE")
         elif isinstance(base_opt, objects.OvsDpdkBond):
             ovs_extra.extend(base_opt.ovs_extra)
             # Referring to bug:1643026, the below commenting of the interfaces,
@@ -307,8 +326,29 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             data += "TYPE=OVSDPDKBond\n"
             data += "OVS_BRIDGE=%s\n" % base_opt.bridge_name
             if base_opt.members:
+                for bond_member in base_opt.members:
+                    # Validation of DPDK port having only one interface is done
+                    # prior to this. So accesing the interface name statically.
+                    # Also pci_address would be valid here, since
+                    # bind_dpdk_interfaces () is invoked before this.
+                    pci_address = utils.get_stored_pci_address(
+                        bond_member.members[0].name, self.noop)
+                    ovs_extra.append("set Interface %s options:"
+                                     "dpdk-devargs=%s"
+                                     % (bond_member.name, pci_address))
                 members = [member.name for member in base_opt.members]
                 data += ("BOND_IFACES=\"%s\"\n" % " ".join(members))
+                # MTU configuration given for the OvsDpdkbond shall be applied
+                # to each of the members of the OvsDpdkbond
+                if base_opt.mtu:
+                    for member in base_opt.members:
+                        ovs_extra.append("set Interface %s mtu_request=$MTU" %
+                                         member.name)
+                if base_opt.rx_queue:
+                    data += "RX_QUEUE=%i\n" % base_opt.rx_queue
+                    for member in base_opt.members:
+                        ovs_extra.append("set Interface %s options:n_rxq="
+                                         "$RX_QUEUE" % member.name)
             if base_opt.ovs_options:
                 data += "OVS_OPTIONS=\"%s\"\n" % base_opt.ovs_options
             ovs_extra.extend(base_opt.ovs_extra)
@@ -358,10 +398,10 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             data += "DHCLIENTARGS=%s\n" % base_opt.dhclient_args
         if base_opt.dns_servers:
             data += "DNS1=%s\n" % base_opt.dns_servers[0]
-            if len(base_opt.dns_servers) == 2:
+            if len(base_opt.dns_servers) >= 2:
                 data += "DNS2=%s\n" % base_opt.dns_servers[1]
-            elif len(base_opt.dns_servers) > 2:
-                logger.warning('ifcfg format supports a max of 2 dns servers.')
+                if len(base_opt.dns_servers) > 2:
+                    logger.warning('ifcfg format supports max 2 resolvers.')
         return data
 
     def _add_routes(self, interface_name, routes=[]):
@@ -626,6 +666,21 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         if ovs_dpdk_bond.routes:
             self._add_routes(ovs_dpdk_bond.name, ovs_dpdk_bond.routes)
 
+    def add_vpp_interface(self, vpp_interface):
+        """Add a VppInterface object to the net config object
+
+        :param vpp_interface: The VppInterface object to add
+        """
+        vpp_interface.pci_dev = utils.get_pci_address(vpp_interface.name,
+                                                      False)
+        vpp_interface.hwaddr = utils.interface_mac(vpp_interface.name)
+        if not self.noop:
+            self.ifdown(vpp_interface.name)
+            remove_ifcfg_config(vpp_interface.name)
+        logger.info('adding vpp interface: %s %s'
+                    % (vpp_interface.name, vpp_interface.pci_dev))
+        self.vpp_interface_data[vpp_interface.name] = vpp_interface
+
     def generate_ivs_config(self, ivs_uplinks, ivs_interfaces):
         """Generate configuration content for ivs."""
 
@@ -690,6 +745,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         restart_bridges = []
         restart_linux_bonds = []
         restart_linux_teams = []
+        restart_vpp = False
         update_files = {}
         all_file_names = []
         ivs_uplinks = []  # ivs physical uplinks
@@ -698,6 +754,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         nfvswitch_internal_ifaces = []  # nfvswitch internal/management ports
         stop_dhclient_interfaces = []
         ovs_needs_restart = False
+        vpp_interfaces = self.vpp_interface_data.values()
 
         for interface_name, iface_data in self.interface_data.items():
             route_data = self.route_data.get(interface_name, '')
@@ -906,6 +963,15 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                 logger.info('No changes required for InfiniBand iface: %s' %
                             interface_name)
 
+        if self.vpp_interface_data:
+            vpp_path = self.root_dir + vpp_config_path()
+            vpp_config = utils.generate_vpp_config(vpp_path, vpp_interfaces)
+            if utils.diff(vpp_path, vpp_config):
+                restart_vpp = True
+                update_files[vpp_path] = vpp_config
+            else:
+                logger.info('No changes required for VPP')
+
         if cleanup:
             for ifcfg_file in glob.iglob(cleanup_pattern()):
                 if ifcfg_file not in all_file_names:
@@ -932,6 +998,9 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             for bridge in restart_bridges:
                 self.ifdown(bridge, iftype='bridge')
 
+            for vpp_interface in vpp_interfaces:
+                self.ifdown(vpp_interface.name)
+
             for oldname, newname in self.renamed_interfaces.items():
                 self.ifrename(oldname, newname)
 
@@ -1012,4 +1081,20 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             for vlan in restart_vlans:
                 self.ifup(vlan)
 
+            if not self.noop:
+                if restart_vpp:
+                    logger.info('Restarting VPP')
+                    utils.restart_vpp(vpp_interfaces)
+
+                if self.vpp_interface_data:
+                    logger.info('Updating VPP mapping')
+                    utils.update_vpp_mapping(vpp_interfaces)
+
+            if self.errors:
+                message = 'Failure(s) occurred when applying configuration'
+                logger.error(message)
+                for e in self.errors:
+                    logger.error('stdout: %s, stderr: %s', e.stdout, e.stderr)
+                raise os_net_config.ConfigurationError(message)
+
         return update_files
