@@ -14,6 +14,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+#
+# NOTE: When making changes to the object model, remember to also update
+#       schema.yaml to reflect changes to the schema of config files!
+#
+
 import logging
 import netaddr
 from oslo_utils import strutils
@@ -25,7 +30,8 @@ logger = logging.getLogger(__name__)
 
 _MAPPED_NICS = None
 
-DEFAULT_OVS_BRIDGE_FAIL_MODE = 'standalone'
+STANDALONE_FAIL_MODE = 'standalone'
+DEFAULT_OVS_BRIDGE_FAIL_MODE = STANDALONE_FAIL_MODE
 
 
 class InvalidConfigException(ValueError):
@@ -68,6 +74,8 @@ def object_from_json(json):
         return OvsDpdkPort.from_json(json)
     elif obj_type == "ovs_dpdk_bond":
         return OvsDpdkBond.from_json(json)
+    elif obj_type == "vpp_interface":
+        return VppInterface.from_json(json)
 
 
 def _get_required_field(json, name, object_name):
@@ -177,6 +185,13 @@ def format_ovs_extra(obj, templates):
     return [t.format(name=obj.name) for t in templates or []]
 
 
+def _add_fail_mode(fail_mode):
+    ovs_extra = ['set bridge {name} fail_mode=%s' % fail_mode]
+    if fail_mode == STANDALONE_FAIL_MODE:
+        ovs_extra.append('del-controller {name}')
+    return ovs_extra
+
+
 class Route(object):
     """Base class for network routes."""
 
@@ -283,7 +298,8 @@ class _BaseOpts(object):
         mtu = json.get('mtu', None)
         dhclient_args = json.get('dhclient_args')
         dns_servers = json.get('dns_servers')
-        nm_controlled = json.get('nm_controlled')
+        nm_controlled = strutils.bool_from_string(str(json.get('nm_controlled',
+                                                      False)))
         primary = strutils.bool_from_string(str(json.get('primary', False)))
         addresses = []
         routes = []
@@ -461,7 +477,7 @@ class OvsBridge(_BaseOpts):
         self.ovs_options = ovs_options
         ovs_extra = ovs_extra or []
         if fail_mode:
-            ovs_extra.append('set bridge {name} fail_mode=%s' % fail_mode)
+            ovs_extra.extend(_add_fail_mode(fail_mode))
         self.ovs_extra = format_ovs_extra(self, ovs_extra)
         for member in self.members:
             member.bridge_name = name
@@ -485,7 +501,9 @@ class OvsBridge(_BaseOpts):
          nm_controlled) = _BaseOpts.base_opts_from_json(
              json, include_primary=False)
         ovs_options = json.get('ovs_options')
-        ovs_extra = json.get('ovs_extra')
+        ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
         fail_mode = json.get('ovs_fail_mode', DEFAULT_OVS_BRIDGE_FAIL_MODE)
 
         members = _update_members(json, nic_mapping, persist_mapping)
@@ -516,7 +534,7 @@ class OvsUserBridge(_BaseOpts):
         self.ovs_options = ovs_options
         ovs_extra = ovs_extra or []
         if fail_mode:
-            ovs_extra.append('set bridge {name} fail_mode=%s' % fail_mode)
+            ovs_extra.extend(_add_fail_mode(fail_mode))
         self.ovs_extra = format_ovs_extra(self, ovs_extra)
         for member in self.members:
             member.bridge_name = name
@@ -542,7 +560,9 @@ class OvsUserBridge(_BaseOpts):
          nm_controlled) = _BaseOpts.base_opts_from_json(
              json, include_primary=False)
         ovs_options = json.get('ovs_options')
-        ovs_extra = json.get('ovs_extra')
+        ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
         fail_mode = json.get('ovs_fail_mode', DEFAULT_OVS_BRIDGE_FAIL_MODE)
 
         members = _update_members(json, nic_mapping, persist_mapping)
@@ -856,6 +876,8 @@ class OvsBond(_BaseOpts):
              json, include_primary=False)
         ovs_options = json.get('ovs_options')
         ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
 
         members = _update_members(json, nic_mapping, persist_mapping)
 
@@ -895,6 +917,8 @@ class OvsTunnel(_BaseOpts):
         ovs_options = json.get('ovs_options', [])
         ovs_options = ['options:%s' % opt for opt in ovs_options]
         ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
         opts = _BaseOpts.base_opts_from_json(json)
         return OvsTunnel(name, *opts, tunnel_type=tunnel_type,
                          ovs_options=ovs_options, ovs_extra=ovs_extra)
@@ -929,6 +953,8 @@ class OvsPatchPort(_BaseOpts):
         ovs_options = json.get('ovs_options', [])
         ovs_options = ['options:%s' % opt for opt in ovs_options]
         ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
         opts = _BaseOpts.base_opts_from_json(json)
         return OvsPatchPort(name, *opts, bridge_name=bridge_name, peer=peer,
                             ovs_options=ovs_options, ovs_extra=ovs_extra)
@@ -966,7 +992,8 @@ class OvsDpdkPort(_BaseOpts):
                  routes=None, mtu=None, primary=False, nic_mapping=None,
                  persist_mapping=False, defroute=True, dhclient_args=None,
                  dns_servers=None, nm_controlled=False, members=None,
-                 driver='vfio-pci', ovs_options=None, ovs_extra=None):
+                 driver='vfio-pci', ovs_options=None, ovs_extra=None,
+                 rx_queue=None):
 
         super(OvsDpdkPort, self).__init__(name, use_dhcp, use_dhcpv6,
                                           addresses, routes, mtu, primary,
@@ -977,6 +1004,7 @@ class OvsDpdkPort(_BaseOpts):
         self.ovs_options = ovs_options or []
         self.ovs_extra = format_ovs_extra(self, ovs_extra)
         self.driver = driver
+        self.rx_queue = rx_queue
 
     @staticmethod
     def from_json(json):
@@ -1018,9 +1046,12 @@ class OvsDpdkPort(_BaseOpts):
             msg = 'DPDK Port should have one member as Interface'
             raise InvalidConfigException(msg)
 
+        rx_queue = json.get('rx_queue', None)
         ovs_options = json.get('ovs_options', [])
         ovs_options = ['options:%s' % opt for opt in ovs_options]
         ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
         return OvsDpdkPort(name, use_dhcp=use_dhcp, use_dhcpv6=use_dhcpv6,
                            addresses=addresses, routes=routes, mtu=mtu,
                            primary=primary, nic_mapping=nic_mapping,
@@ -1029,7 +1060,7 @@ class OvsDpdkPort(_BaseOpts):
                            dns_servers=dns_servers,
                            nm_controlled=nm_controlled, members=members,
                            driver=driver, ovs_options=ovs_options,
-                           ovs_extra=ovs_extra)
+                           ovs_extra=ovs_extra, rx_queue=rx_queue)
 
 
 class OvsDpdkBond(_BaseOpts):
@@ -1039,7 +1070,7 @@ class OvsDpdkBond(_BaseOpts):
                  routes=None, mtu=None, primary=False, members=None,
                  ovs_options=None, ovs_extra=None, nic_mapping=None,
                  persist_mapping=False, defroute=True, dhclient_args=None,
-                 dns_servers=None, nm_controlled=False):
+                 dns_servers=None, nm_controlled=False, rx_queue=None):
         super(OvsDpdkBond, self).__init__(name, use_dhcp, use_dhcpv6,
                                           addresses, routes, mtu, primary,
                                           nic_mapping, persist_mapping,
@@ -1048,6 +1079,7 @@ class OvsDpdkBond(_BaseOpts):
         self.members = members or []
         self.ovs_options = ovs_options
         self.ovs_extra = format_ovs_extra(self, ovs_extra)
+        self.rx_queue = rx_queue
 
         for member in self.members:
             if member.primary:
@@ -1070,8 +1102,11 @@ class OvsDpdkBond(_BaseOpts):
          persist_mapping, defroute, dhclient_args,
          dns_servers, nm_controlled) = _BaseOpts.base_opts_from_json(
              json, include_primary=False)
+        rx_queue = json.get('rx_queue', None)
         ovs_options = json.get('ovs_options')
         ovs_extra = json.get('ovs_extra', [])
+        if not isinstance(ovs_extra, list):
+            ovs_extra = [ovs_extra]
         members = []
 
         # members
@@ -1099,4 +1134,58 @@ class OvsDpdkBond(_BaseOpts):
                            persist_mapping=persist_mapping,
                            defroute=defroute, dhclient_args=dhclient_args,
                            dns_servers=dns_servers,
-                           nm_controlled=nm_controlled)
+                           nm_controlled=nm_controlled, rx_queue=rx_queue)
+
+
+class VppInterface(_BaseOpts):
+    """Base class for VPP Interface.
+
+    Vector Packet Processing (VPP) is a high performance packet processing
+    stack that runs in user space in Linux. VPP is used as an alternative to
+    kernel networking stack for accelerated network data path. VPP uses DPDK
+    poll-mode drivers to bind system interfaces rather than kernel drivers.
+    VPP bound interfaces are not visible to kernel networking stack, so we
+    must handle them separately.
+
+    The following parameters can be specified in addition to base Interface:
+      - uio_driver: DPDK poll-mode driver name. Defaults to 'vfio-pci', valid
+                    values are 'uio_pci_generic' and 'vfio-pci'.
+      - options: Interface options such as vlan stripping and tx/rx transmit
+                 queues specification. Defaults to None. Reference for those
+                 configurations can be found at
+                 https://wiki.fd.io/view/VPP/Command-line_Arguments
+                 Example: 'vlan-strip-offload on num-rx-queues 3'
+
+    Note that 'name' attribute is used to indicate the kernel nic that should
+    be bound to VPP. Once VPP binds the interface, a mapping file will be
+    updated with the interface's information, and this file will be used in
+    subsequent runs of os-net-config.
+    """
+    def __init__(self, name, use_dhcp=False, use_dhcpv6=False, addresses=None,
+                 routes=None, mtu=None, primary=False, nic_mapping=None,
+                 persist_mapping=False, defroute=True, dhclient_args=None,
+                 dns_servers=None, nm_controlled=False, uio_driver='vfio-pci',
+                 options=None):
+        addresses = addresses or []
+
+        super(VppInterface, self).__init__(name, use_dhcp, use_dhcpv6,
+                                           addresses, routes, mtu, primary,
+                                           nic_mapping, persist_mapping,
+                                           defroute, dhclient_args,
+                                           dns_servers, nm_controlled)
+        self.uio_driver = uio_driver
+        self.options = options
+        # pci_dev contains pci address for the interface, it will be populated
+        # when interface is added to config object. It will be determined
+        # either through ethtool or by looking up the dpdk mapping file.
+        self.pci_dev = None
+
+    @staticmethod
+    def from_json(json):
+        name = _get_required_field(json, 'name', 'VppInterface')
+        uio_driver = json.get('uio_driver', 'vfio-pci')
+        options = json.get('options', '')
+
+        opts = _BaseOpts.base_opts_from_json(json)
+        return VppInterface(name, *opts, uio_driver=uio_driver,
+                            options=options)
