@@ -18,6 +18,7 @@ import tempfile
 
 from oslo_concurrency import processutils
 
+import os_net_config
 from os_net_config import impl_eni
 from os_net_config import objects
 from os_net_config.tests import base
@@ -72,8 +73,9 @@ iface br0 inet dhcp
     pre-up ip addr flush dev eth0
 """
 
-_OVS_BRIDGE_DHCP_STANDALONE = _OVS_BRIDGE_DHCP + \
-    "    ovs_extra set bridge br0 fail_mode=standalone\n"
+_OVS_BRIDGE_DHCP_STANDALONE = _OVS_BRIDGE_DHCP + (
+    "    ovs_extra set bridge br0 fail_mode=standalone "
+    "-- del-controller br0\n")
 
 _OVS_BRIDGE_DHCP_SECURE = _OVS_BRIDGE_DHCP + \
     "    ovs_extra set bridge br0 fail_mode=secure\n"
@@ -360,3 +362,33 @@ class TestENINetConfigApply(base.TestCase):
         self.assertEqual((_OVS_BRIDGE_DHCP + _OVS_PORT_IFACE), iface_data)
         self.assertIn('eth0', self.ifup_interface_names)
         self.assertIn('br0', self.ifup_interface_names)
+
+    def _failed_execute(*args, **kwargs):
+        if kwargs.get('check_exit_code', True):
+            raise processutils.ProcessExecutionError('Test stderr',
+                                                     'Test stdout',
+                                                     str(kwargs))
+
+    def test_interface_failure(self):
+        self.stubs.Set(processutils, 'execute', self._failed_execute)
+        v4_addr = objects.Address('192.168.1.2/24')
+        interface = objects.Interface('em1', addresses=[v4_addr])
+        self.provider.add_interface(interface)
+
+        self.assertRaises(os_net_config.ConfigurationError,
+                          self.provider.apply)
+        self.assertEqual(1, len(self.provider.errors))
+
+    def test_interface_failure_multiple(self):
+        self.stubs.Set(processutils, 'execute', self._failed_execute)
+        v4_addr = objects.Address('192.168.1.2/24')
+        interface = objects.Interface('em1', addresses=[v4_addr])
+        v4_addr2 = objects.Address('192.168.2.2/24')
+        interface2 = objects.Interface('em2', addresses=[v4_addr2])
+        self.provider.add_interface(interface)
+        self.provider.add_interface(interface2)
+
+        self.assertRaises(os_net_config.ConfigurationError,
+                          self.provider.apply)
+        # Even though the first one failed, we should have attempted both
+        self.assertEqual(2, len(self.provider.errors))
