@@ -144,6 +144,46 @@ class TestInterface(base.TestCase):
         interface1 = objects.object_from_json(json.loads(data))
         self.assertEqual("--foobar", interface1.dhclient_args)
 
+    def test_from_json_nm_controlled_false(self):
+        data = """{
+"type": "interface",
+"name": "em1",
+"nm_controlled": false
+}
+"""
+        interface1 = objects.object_from_json(json.loads(data))
+        self.assertFalse(interface1.nm_controlled)
+
+    def test_from_json_nm_controlled_true(self):
+        data = """{
+"type": "interface",
+"name": "em1",
+"nm_controlled": true
+}
+"""
+        interface1 = objects.object_from_json(json.loads(data))
+        self.assertTrue(interface1.nm_controlled)
+
+    def test_from_json_nm_controlled_false_boolstr(self):
+        data = """{
+"type": "interface",
+"name": "em1",
+"nm_controlled": "no"
+}
+"""
+        interface1 = objects.object_from_json(json.loads(data))
+        self.assertFalse(interface1.nm_controlled)
+
+    def test_from_json_nm_controlled_true_boolstr(self):
+        data = """{
+"type": "interface",
+"name": "em1",
+"nm_controlled": "yes"
+}
+"""
+        interface1 = objects.object_from_json(json.loads(data))
+        self.assertTrue(interface1.nm_controlled)
+
     def test_from_json_dns_servers(self):
         data = """{
 "type": "interface",
@@ -295,6 +335,35 @@ class TestBridge(base.TestCase):
         self.assertTrue(interface2.ovs_port)
         self.assertEqual("br-foo", interface2.bridge_name)
 
+    def test_from_json_ovs_extra(self):
+        data = """{
+"type": "ovs_bridge",
+"name": "br-foo",
+"ovs_extra": ["bar"],
+"ovs_fail_mode": "standalone"
+}
+"""
+        bridge = objects.object_from_json(json.loads(data))
+        self.assertTrue(3 == len(bridge.ovs_extra))
+        self.assertItemsEqual(["bar",
+                               "set bridge br-foo fail_mode=standalone",
+                               "del-controller br-foo"],
+                              bridge.ovs_extra)
+
+    def test_from_json_ovs_extra_string(self):
+        data = """{
+"type": "ovs_bridge",
+"name": "br-foo",
+"ovs_extra": "bar",
+"ovs_fail_mode": "standalone"
+}
+"""
+        bridge = objects.object_from_json(json.loads(data))
+        self.assertItemsEqual(["bar",
+                               "set bridge br-foo fail_mode=standalone",
+                               "del-controller br-foo"],
+                              bridge.ovs_extra)
+
 
 class TestLinuxBridge(base.TestCase):
 
@@ -702,7 +771,8 @@ class TestOvsTunnel(base.TestCase):
         bridge = objects.object_from_json(json.loads(data))
         self.assertEqual("br-foo", bridge.name)
         self.assertEqual(["set bridge br-foo something",
-                          "set bridge br-foo fail_mode=standalone"],
+                          "set bridge br-foo fail_mode=standalone",
+                          "del-controller br-foo"],
                          bridge.ovs_extra)
         tun0 = bridge.members[0]
         self.assertEqual("tun0", tun0.name)
@@ -1100,3 +1170,19 @@ class TestOvsDpdkBond(base.TestCase):
         self.assertEqual("vfio-pci", dpdk_port1.driver)
         iface2 = dpdk_port1.members[0]
         self.assertEqual("eth2", iface2.name)
+
+
+class TestVppInterface(base.TestCase):
+    def test_vpp_interface_from_json(self):
+        data = """{
+"type": "vpp_interface",
+"name": "em1",
+"uio_driver": "uio_pci_generic",
+"options": "vlan-strip-offload off"
+}
+"""
+
+        vpp_interface = objects.object_from_json(json.loads(data))
+        self.assertEqual("em1", vpp_interface.name)
+        self.assertEqual("uio_pci_generic", vpp_interface.uio_driver)
+        self.assertEqual("vlan-strip-offload off", vpp_interface.options)
